﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.CheckService.Services;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Services;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Goods;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsTechnology;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsType;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;
using Microsoft.AspNetCore.Components.Forms;
using Newtonsoft.Json;
using SqlSugar;
using System.Linq;
using GoodsDto = BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos.GoodsDto;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Services
{
    public class GoodsService : IGoodsService, ITransient
    {
        private readonly ICheckServices _checkServices;
        private readonly IThirdpartyPushService _thirdpartyPushService;
        public GoodsService(ICheckServices checkServices, IThirdpartyPushService thirdpartyPushService)
        {
            _checkServices = checkServices;
            _thirdpartyPushService = thirdpartyPushService;
        }
        /// <summary>
        /// 分页查询商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<GoodsDto>>> GetGoodsPageList(GoodsPageInputDto inputDto)
        {
            int total = new RefAsync<int>();
            var data = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo, BPA_GoodsType, BPA_GoodsUint>((a, b, c) =>
                new JoinQueryInfos(JoinType.Left, a.GoodsTypeId == b.Id,
                    JoinType.Left, a.GoodsUintId == c.Id))
                .Where((a, b, c) => a.IsDeleted == 0)
                .WhereIF(!string.IsNullOrEmpty(inputDto.GoodsTypeName), (a, b, c) => b.Name.Contains(inputDto.GoodsTypeName))
                .Select((a, b, c) => new GoodsDto()
                {
                    Id = a.Id.SelectAll(),
                    GoodsTypeName = b.Name,
                    GoodsUnitName = c.Name

                })
                .Mapper(x =>
                {
                    var GoodsAttributePriceList = SqlSugarDb.Db.Queryable<BPA_GoodsAttributePrice>().Where(y => y.GoodsId == x.Id).ToList();
                    var list = GoodsAttributePriceList.Adapt<List<GoodsAttributePrice>>();
                    x.GoodsAttributePrice = list;
                })
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);
            return new PageUtil<List<GoodsDto>>()
            {
                Total = total,
                Data = data
            };
        }
        /// <summary>
        /// 添加商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<ResponseMessageDto>> AddGoods(List<GoodsInsertDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var goodsList = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
                var typeList = await SqlSugarDb.Db.Queryable<BPA_GoodsType>().ToListAsync();
                var uintList = await SqlSugarDb.Db.Queryable<BPA_GoodsUint>().ToListAsync();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var defaultUintData = uintList.FirstOrDefault(t => t.Name == "份");
                var defaultUint = new DefaultUintDto();
                if (defaultUintData != null)
                {
                    defaultUint.Id = defaultUintData.Id;
                    defaultUint.Name = defaultUintData.Name;
                }
                else
                {
                    var dUint = new BPA_GoodsUint
                    {
                        Id = Guid.NewGuid().ToString(),
                        Name = "份",
                        CreateBy = userId
                    };
                    await SqlSugarDb.Db.Insertable(dUint).ExecuteCommandAsync();
                    defaultUint.Id = dUint.Id;
                    defaultUint.Name = dUint.Name;
                }
                var goodAddList = new List<BPA_GoodsInfo>();
                var goodUpdateList = new List<BPA_GoodsInfo>();
                foreach (var item in dto)
                {
                    var goodData = goodsList.FirstOrDefault(t => t.Name == item.Name && t.Id != item.Id);
                    if (goodData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "商品上传失败：重复商品数据", Id = item.Id, Name = item.Name });
                    var typeData = typeList.FirstOrDefault(t => t.Id == item.GoodsTypeId);
                    if (typeData == null)
                        repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "工艺分类上传失败：未找到工艺分类数据", Id = item.GoodsTypeId, Name = item.GoodsTypeName });
                    if (string.IsNullOrWhiteSpace(item.GoodsUintId))
                    {
                        item.GoodsUintId = defaultUint.Id;
                        item.GoodsUintName = defaultUint.Name;
                    }
                    else
                    {
                        var uintData = uintList.FirstOrDefault(t => t.Id == item.GoodsUintId);
                        if (uintData == null)
                            repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品单位上传失败：未找到商品单位数据", Id = item.GoodsUintId, Name = item.GoodsUintName });
                    }
                    var goodInfo = goodsList.FirstOrDefault(t => t.Id == item.Id);
                    if (goodInfo == null)
                    {
                        goodInfo = new BPA_GoodsInfo
                        {
                            Id = item.Id,
                            Name = item.Name,
                            GoodsTypeId = item.GoodsTypeId,
                            Descritption = item.Descritption,
                            ImgUrl = item.ImgUrl,
                            ImgMode = 0,
                            Sort = item.Sort,
                            Key = null,
                            Code = GetNumber2(8),
                            Price = item.Price,
                            CreateBy = userId,
                            GoodsUintId = item.GoodsUintId,
                            IsWeigh = item.IsWeigh,
                            ForeignKeyRe = null,
                            Design = null,
                            DefaultMate = null,
                            IsAttrubute = true
                        };
                        goodAddList.Add(goodInfo);
                    }
                    else
                    {
                        goodInfo.Name = item.Name;
                        goodInfo.GoodsTypeId = item.GoodsTypeId;
                        goodInfo.Descritption = item.Descritption;
                        goodInfo.ImgUrl = item.ImgUrl;
                        goodInfo.Sort = item.Sort;
                        goodInfo.Price = item.Price;
                        goodInfo.GoodsUintId = item.GoodsUintId;
                        goodInfo.IsWeigh = item.IsWeigh;
                        goodUpdateList.Add(goodInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "商品上传成功", Id = item.Id, Name = item.Name });
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Insertable(goodAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(goodUpdateList).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return successList;
                #region 下发数据到设备（后厨忽略）

                //if (dto.IsPush)
                //{
                //    var data = SqlSugarDb.Db.Queryable<BPA_GoodsInfo, BPA_GoodsType, BPA_GoodsUint>((a, b, c) =>
                //new JoinQueryInfos(JoinType.Left, a.GoodsTypeId == b.Id,
                //    JoinType.Left, a.GoodsUintId == c.Id))
                //.Where((a, b, c) => a.Id == res.Id).Select((a, b, c) => new GoodsInfoPushDto()
                //{
                //    Id = a.Id,
                //    Name = a.Name,
                //    Descritption = a.Descritption,
                //    ImgUrl = a.ImgUrl,
                //    Sort = a.Sort,
                //    Price = a.Price,
                //    GoodsUintId = a.GoodsUintId,
                //    GoodsUintName = c.Name,
                //    IsWeigh = a.IsWeigh,
                //    Design = a.Design,
                //    DefaultMate = a.DefaultMate,
                //    IsAttrubute = a.IsAttrubute,
                //    GoodsTypeId = a.GoodsTypeId,
                //    GoodsTypeName = b.Name,
                //}).First();
                //    await _thirdpartyPushService.AddPushRecordAndPushDevice(dto, 1, res.Id,
                //        JsonConvert.SerializeObject(data));
                //}
                #endregion
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 更新商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoods(List<GoodsUpdateDto> dto)
        {
            var goodList = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
            var typeList = await SqlSugarDb.Db.Queryable<BPA_GoodsType>().ToListAsync();
            var uintList = await SqlSugarDb.Db.Queryable<BPA_GoodsUint>().ToListAsync();
            var list = new List<BPA_GoodsInfo>();
            foreach (var item in dto)
            {
                var typeData = typeList.FirstOrDefault(x => x.Id == item.GoodsTypeId);
                if (typeData == null)
                    throw Oops.Oh($"工艺分类“{item.GoodsTypeName}”不存在");
                var uintData = uintList.FirstOrDefault(x => x.Id == item.GoodsUintId);
                if (uintData == null)
                    throw Oops.Oh($"商品单位“{item.GoodsUintName}”不存在");
                var resEntity = goodList.FirstOrDefault(it => it.Id == item.Id);
                if (resEntity == null)
                    throw Oops.Oh($"商品“{item.Name}”不存在！");
                resEntity.Id = item.Id;
                resEntity.GoodsTypeId = item.GoodsTypeId;
                resEntity.Name = item.Name;
                resEntity.Descritption = item.Descritption;
                resEntity.ImgUrl = item.ImgUrl;
                resEntity.Price = item.Price;
                resEntity.IsWeigh = item.IsWeigh;
                resEntity.GoodsUintId = item.GoodsUintId;
                list.Add(resEntity);

            }
            var res = await SqlSugarDb.Db.Updateable(list).ExecuteCommandAsync();
            #region 下发数据到设备（后厨忽略）

            //if (dto.IsPush)
            //{
            //    var data = SqlSugarDb.Db.Queryable<BPA_GoodsInfo, BPA_GoodsType, BPA_GoodsUint>((a, b, c) =>
            // new JoinQueryInfos(JoinType.Left, a.GoodsTypeId == b.Id,
            //     JoinType.Left, a.GoodsUintId == c.Id))
            // .Where((a, b, c) => a.Id == resEntity.Id).Select((a, b, c) => new GoodsInfoPushDto()
            // {
            //     Id = a.Id,
            //     Name = a.Name,
            //     Descritption = a.Descritption,
            //     ImgUrl = a.ImgUrl,
            //     Sort = a.Sort,
            //     Price = a.Price,
            //     GoodsUintId = a.GoodsUintId,
            //     GoodsUintName = c.Name,
            //     IsWeigh = a.IsWeigh,
            //     Design = a.Design,
            //     DefaultMate = a.DefaultMate,
            //     IsAttrubute = a.IsAttrubute,
            //     GoodsTypeId = a.GoodsTypeId,
            //     GoodsTypeName = b.Name,
            // }).First();
            //    await _thirdpartyPushService.AddPushRecordAndPushDevice(dto, 1, resEntity.Id,
            //        JsonConvert.SerializeObject(data));
            //}
            #endregion
            return res > 0;
        }
        /// <summary>
        /// 删除商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> DeleteGoods(DeleteGoodsDto dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var goods = SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().Where(x => dto.Ids.Contains(x.Id)).ToList();
                if (goods == null) throw Oops.Oh(ErrorCodeEnum.Code1008);
                var goodsbom = SqlSugarDb.Db.Queryable<BPA_GoodsBom>().Where(x => dto.Ids.Contains( x.Goods_Id)).ToList();
                var goodsTechnology = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => dto.Ids.Contains(x.GoodsId)).ToList();
                await SqlSugarDb.Db.Deleteable(goodsbom).ExecuteCommandAsync();
                await SqlSugarDb.Db.Deleteable(goodsTechnology).ExecuteCommandAsync();
                var res = await SqlSugarDb.Db.Deleteable(goods).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }

        }
       /// <summary>
       /// 添加商品属性
       /// </summary>
       /// <param name="dto"></param>
       /// <returns></returns>
        public async Task<List<ResponseMessageDto>> AddGoodsAttribute(List<GoodsAttributeInsertDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var typeList = await SqlSugarDb.Db.Queryable<BPA_GoodsType>().ToListAsync();
                var attrList = await SqlSugarDb.Db.Queryable<BPA_GoodsAttribute>().ToListAsync();
                var attrValueList = await SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var attrAddList = new List<BPA_GoodsAttribute>();
                var attrUpdateList = new List<BPA_GoodsAttribute>();
                var attrValueAddList = new List<BPA_GoodsAttributeValue>();
                var attrValueUpdateList = new List<BPA_GoodsAttributeValue>();
                foreach (var item in dto)
                {
                    var typeData = typeList.FirstOrDefault(t => t.Id == item.GoodsTypeId);
                    if(typeData == null)
                        repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "分类属性上传失败：未找到工艺分类数据", Id = item.GoodsTypeId, Name = item.GoodsTypeName });
                    var attrData = attrList.FirstOrDefault(t => t.AttributeName == item.AttributeName && t.Id != item.Id && t.GoodsTypeId == item.GoodsTypeId);
                    if(attrData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "分类属性上传失败：重复属性数据", Id = item.Id, Name = item.AttributeName });
                    var attrInfo = attrList.FirstOrDefault(t => t.Id == item.Id);
                    if(attrInfo == null)
                    {
                        attrInfo = new BPA_GoodsAttribute
                        {
                            Id = item.Id,
                            AttributeName = item.AttributeName,
                            GoodsTypeId = item.GoodsTypeId,
                            CreateBy = userId,
                            Sort = item.Sort
                        };
                        attrAddList.Add(attrInfo);
                    }
                    else
                    {
                        attrInfo.AttributeName = item.AttributeName;
                        attrInfo.GoodsTypeId = item.GoodsTypeId;
                        attrInfo.Sort = item.Sort;
                        attrUpdateList.Add(attrInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "分类属性上传成功", Id = item.Id, Name = item.AttributeName });
                    if (item.GoodsAttributeValueList == null)
                        item.GoodsAttributeValueList = new List<GoodsAttributeInsertValue>();
                    foreach (var value in item.GoodsAttributeValueList)
                    {
                        var valueData = attrValueList.FirstOrDefault(t => t.AttributeValue == value.AttributeValue && t.Id != value.Id && t.GoodsAttributeId == item.Id);
                        if(valueData != null)
                            repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "分类属性上传失败：重复属性值数据", Id = value.Id, Name = value.AttributeValue });
                        var attrValueInfo = attrValueList.FirstOrDefault(t => t.Id == value.Id);
                        if(attrValueInfo == null)
                        {
                            attrValueInfo = new BPA_GoodsAttributeValue
                            {
                                Id = value.Id,
                                GoodsAttributeId = item.Id,
                                AttributeValue = value.AttributeValue,
                                CreateBy = userId,
                                Sort = item.Sort,
                                WaiKey = null
                            };
                            attrValueAddList.Add(attrValueInfo);
                        }
                        else
                        {
                            attrValueInfo.GoodsAttributeId = item.Id;
                            attrValueInfo.AttributeValue = value.AttributeValue;
                            attrValueInfo.Sort = item.Sort;
                            attrValueUpdateList.Add(attrValueInfo);
                            await SqlSugarDb.Db.Updateable(attrValueInfo).ExecuteCommandAsync();
                        }
                        successList.Add(new ResponseMessageDto { Code = 30000, Message = "分类属性值上传成功", Id = item.Id, Name = item.AttributeName });
                    }
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Insertable(attrAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(attrUpdateList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Insertable(attrValueAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(attrValueUpdateList).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }  
        }
        public async Task<bool> AddGoodsAttributeValue(List<GoodsAttributeValue> dto)
        {
            try
            {
                List<BPA_GoodsAttributeValue> GoodsAttributeValueList = new();
                List<BPA_GoodsAttributeValue> GoodsAttributeValueListup = new();
                if (dto.Count > 0)
                {
                    for (int i = 0; i < dto.Count; i++)
                    {
                        if (string.IsNullOrWhiteSpace(dto[i].Id))
                        {
                            BPA_GoodsAttributeValue _GoodsAttributeValue = new();
                            _GoodsAttributeValue.Id = Guid.NewGuid().ToString();
                            _GoodsAttributeValue.GoodsAttributeId = dto[i].GoodsAttributeId;
                            _GoodsAttributeValue.AttributeValue = dto[i].AttributeValue;
                            _GoodsAttributeValue.Sort = dto[i].Sort;
                            GoodsAttributeValueList.Add(_GoodsAttributeValue);
                        }
                        else
                        {
                            var _GoodsAttributeValue=SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().Where(x => x.Id == dto[i].Id).First();
                            if (_GoodsAttributeValue != null)
                            {
                              //  _GoodsAttributeValue.Id = Guid.NewGuid().ToString();
                                _GoodsAttributeValue.GoodsAttributeId = dto[i].GoodsAttributeId;
                                _GoodsAttributeValue.AttributeValue = dto[i].AttributeValue;
                                _GoodsAttributeValue.Sort = dto[i].Sort;
                                GoodsAttributeValueListup.Add(_GoodsAttributeValue);
                            }
                        }
                    }
                }
                var res = 0;
                if (GoodsAttributeValueListup.Count > 0)
                {
                     res= await SqlSugarDb.Db.Updateable(GoodsAttributeValueListup).ExecuteCommandAsync();
                }
                if (GoodsAttributeValueList.Count > 0)
                {
                     res = await SqlSugarDb.Db.Insertable(GoodsAttributeValueList).ExecuteCommandAsync();
                }
                return res > 0;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 查询商品属性
        /// </summary>
        /// <returns></returns>
        public async Task<List<GoodsAttributeVewDto>> GetGoodsAttributeList(GoodsIdDto dto)
        {
            var goodstypeid = "";
            var goods=SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().Where(x => x.Id == dto.GoodsId).First();
            if (goods != null)
            {
                goodstypeid = goods.GoodsTypeId;
            }
            if (!string.IsNullOrWhiteSpace(dto.GoodsTypeId)) goodstypeid = dto.GoodsTypeId;
            var res = await SqlSugarDb.Db.Queryable<BPA_GoodsAttribute>()
                .WhereIF(!string.IsNullOrEmpty(goodstypeid),x=>x.GoodsTypeId== goodstypeid)
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select(a => new GoodsAttributeVewDto
                {
                    Id = a.Id,
                    AttributeName = a.AttributeName,
                    GoodsTypeId = a.GoodsTypeId,
                    Sort = a.Sort,
                })
                .Mapper(x =>
                {
                    var list = SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().Where(p => p.GoodsAttributeId == x.Id).Select(x=>new GoodsAttributeValue() { AttributeValue=x.AttributeValue,Id=x.Id, GoodsAttributeId =x.GoodsAttributeId}).ToList();
                    x.AttributeValueList = list;
                })
                .ToListAsync();
            return res;
        }
        /// <summary>
        /// 查询商品工艺
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<GoodsTechnologyActionListViewDto>> GetGoodsTechnologyAction(GoodsIdDto dto)
        {
            var goodsTechnologyActionListViews = new List<GoodsTechnologyActionListViewDto>();
            var list = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => x.GoodsId == dto.GoodsId)
                .Select(x => new GoodsTechnologyActionViewDto()
                {
                    Id = x.Id.SelectAll(),
                }).OrderBy(x => x.Sort, OrderByType.Asc).ToList();
            var Devicelist = await SqlSugarDb.Db.Queryable<BPA_DeviceInfo>().ToListAsync();
            var goodlist = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
            var GoodsAttributeValue = await SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
            var sf = list.GroupBy(x => (x.GoodsAttributeId, x.WarehousrTemplateId)).ToList();
            for (int i = 0; i < sf.Count; i++)
            {
                var sd = GoodsAttributeValue?.Where(x => sf[i].Key.GoodsAttributeId.Contains(x.Id)).OrderBy(x => sf[i].Key.GoodsAttributeId.IndexOf(x.Id)).Select(x => x.AttributeValue).ToArray();
                var sdw = GoodsAttributeValue?.Where(x => sf[i].Key.GoodsAttributeId.Contains(x.Id)).OrderBy(x => sf[i].Key.GoodsAttributeId.IndexOf(x.Id)).Select(x => x.Id).ToArray();
                var nane = string.Join('-', sd);
                var s = sf[i].AsQueryable().ToList().GroupBy(x => x.DeviceId).ToList();
                for (int t = 0; t < s.Count; t++)
                {
                    var ss = s[t];
                    GoodsTechnologyActionListViewDto item = new()
                    {
                        DeviceId = s[t].Key,
                        WarehousrTemplateId = sf[i].Key.WarehousrTemplateId,
                        GoodsAttributeId = string.Join(',', sdw),
                        TechnologyName = Devicelist?.FirstOrDefault(x => x.Id == s[t].Key)?.DeviceName + "-" + goodlist?.FirstOrDefault(x => x.Id == dto.GoodsId)?.Name + "【" + nane + "】",
                        Data = s[t].AsQueryable().ToList(),
                    };
                    goodsTechnologyActionListViews.Add(item);
                }
            }
            return goodsTechnologyActionListViews;
        }

        public async Task<List<ResponseMessageDto>> AddGoodsTechnologyAction(List<GoodsTechnologyActionUpdateDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var goodsList = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
                var deviceList = await SqlSugarDb.Db.Queryable<BPA_DeviceInfo>().ToListAsync();
                var warehouseTemplateList = await SqlSugarDb.Db.Queryable<BPA_WarehouseTemplate>().ToListAsync();
                var goodsTechnologyActionList = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().ToList();
                var technologyList = await SqlSugarDb.Db.Queryable<BPA_Technology>().ToListAsync();
                var technologyActionList = await SqlSugarDb.Db.Queryable<BPA_TechnologyAction>().ToListAsync();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var actionAddList = new List<BPA_GoodsTechnologyAction>();
                var actionDeleteList = new List<BPA_GoodsTechnologyAction>();
                foreach (var item in dto)
                {
                    var goodsData = goodsList.FirstOrDefault(t => t.Id == item.GoodsId);
                    if (goodsData == null)
                        repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品工艺上传失败：未找到商品数据", Id = item.GoodsId, Name = item.GoodsName });
                    var deviceData = deviceList.FirstOrDefault(t => t.AutoKey == item.DeviceId);
                    if (deviceData == null)
                        repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品工艺上传失败：未找到设备数据", Id = item.DeviceId.ToString(), Name = item.DeviceName });
                    var warehouseId = string.Empty;
                    if (string.IsNullOrWhiteSpace(item.WarehouseTemplateId))
                    {
                        warehouseId = warehouseTemplateList.Where(t => t.DeviceId == deviceData?.Id).OrderBy(a => a.Sort).FirstOrDefault()?.Id;
                    }
                    else
                    {
                        var warehouseTemplateData = warehouseTemplateList.FirstOrDefault(t => t.Id == item.WarehouseTemplateId);
                        if(warehouseTemplateData == null)
                            repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品工艺上传失败：未找到设备仓位数据", Id = item.WarehouseTemplateId, Name = item.WarehouseTemplateName });
                        warehouseId = item.WarehouseTemplateId;
                    }
                    var order = 1;
                    var attrIdGroup = item.TechnologyActionList.GroupBy(t => t.GoodsAttributeId).ToList();
                    foreach (var attrId in attrIdGroup)
                    {
                        var oldActionList = goodsTechnologyActionList.Where(t => t.GoodsId == item.GoodsId && t.DeviceId == deviceData.Id && t.GoodsAttributeId == attrId.Key && t.WarehousrTemplateId == warehouseId).ToList();
                        actionDeleteList.AddRange(oldActionList);
                        foreach (var action in attrId)
                        {
                            var technologyData = technologyList.FirstOrDefault(t => t.Id == action.ChnologyId);
                            if (technologyData == null)
                                repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品工艺上传失败：未找到工艺数据", Id = action.ChnologyId, Name = action.StepName });
                            foreach (var json in action.ActionJson)
                            {
                                var technologyActionData = technologyActionList.FirstOrDefault(t => t.Id == json.technologyactionId);
                                if (technologyActionData == null)
                                    repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品工艺上传失败：未找到工艺值数据", Id = json.technologyactionId, Name = json.actionName });
                            }
                            var actionInfo = new BPA_GoodsTechnologyAction
                            {
                                Id = action.Id,
                                StepName = action.StepName,
                                ChnologyId = action.ChnologyId,
                                GoodsAttributeId = action.GoodsAttributeId,
                                Sort = action.Sort,
                                CreateBy = userId,
                                IsDeleted = 0,
                                ActionJson = JsonConvert.SerializeObject(action.ActionJson),
                                IsBatch = false,
                                GoodsId = item.GoodsId,
                                DeviceId = deviceData.Id,
                                WarehousrTemplateId = warehouseId,
                                Order = order
                            };
                            actionAddList.Add(actionInfo);
                            successList.Add(new ResponseMessageDto { Code = 30000, Message = "商品工艺上传成功", Id = action.Id, Name = action.StepName });
                        }
                        order++;
                    }
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Deleteable(actionDeleteList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Insertable(actionAddList).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        public async Task<bool> DeleteGoodsTechnologyAction(GoodsTechnologyActionDelDto dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                foreach (var id in dto.Ids)
                {
                    var item = await SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => x.Id == id).FirstAsync();
                    if (item != null)
                        throw Oops.Oh("工艺不存在");
                    var res = await SqlSugarDb.Db.Deleteable(item).ExecuteCommandAsync() > 0;
                    var list = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => x.GoodsId == item.GoodsId && x.DeviceId == item.DeviceId).OrderBy(x => x.Sort, OrderByType.Asc).ToList();
                    var uplist = new List<BPA_GoodsTechnologyAction>();
                    for (int i = 0; i < list.Count; i++)
                    {
                        list[i].Sort = i + 1;
                        uplist.Add(list[i]);
                    }
                    if (uplist.Count > 0)
                        await SqlSugarDb.Db.Updateable(uplist).ExecuteCommandAsync();
                    SqlSugarDb.Db.CommitTran();
                }
                return true;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        public async Task<PageUtil> GetGoodsTypePage(GoodstypeQueryDto dto)
        {
            int total = new RefAsync<int>();
            var data = await SqlSugarDb.Db.Queryable<BPA_GoodsType>()
                .Select(a => new GoodstypeViewDto()
                {
                    Id = a.Id.SelectAll()
                })
                .ToPageListAsync(dto.Current, dto.PageSize, total);
            return new PageUtil()
            {
                Total = total,
                Data = data
            };
        }
        public async Task<List<ResponseMessageDto>> AddGoodsType(List<GoodstypeInsertDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var typeList = await SqlSugarDb.Db.Queryable<BPA_GoodsType>().ToListAsync();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var typeAddList = new List<BPA_GoodsType>();
                var typeUpdateList = new List<BPA_GoodsType>();
                foreach (var item in dto)
                {
                    var typeData = typeList.FirstOrDefault(t => t.Name == item.Name && t.Id != item.Id);
                    if(typeData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "工艺分类上传失败：重复工艺分类数据", Id = item.Id, Name = item.Name });
                    if(!string.IsNullOrWhiteSpace(item.Pid))
                    {
                        var pidData = typeList.FirstOrDefault(t => t.Id == item.Pid);
                        if(pidData == null)
                            repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "工艺分类上传失败：未找到工艺分类父级数据", Id = item.Pid, Name = null });
                    }
                    var typeInfo = typeList.FirstOrDefault(t => t.Id == item.Id);
                    if (typeInfo == null)
                    {
                        typeInfo = new BPA_GoodsType
                        {
                            Id = item.Id,
                            Name = item.Name,
                            Sort = item.Sort,
                            Remark = item.Remark,
                            CreateBy = userId,
                            Pid = string.IsNullOrWhiteSpace(item.Pid) ? "0" : item.Pid,
                        };
                        typeAddList.Add(typeInfo);
                    }
                    else
                    {
                        typeInfo.Name = item.Name;
                        typeInfo.Sort = item.Sort;
                        typeInfo.Remark = item.Remark;
                        typeInfo.Pid = string.IsNullOrWhiteSpace(item.Pid) ? "0" : item.Pid;
                        typeUpdateList.Add(typeInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "工艺分类上传成功", Id = item.Id, Name = item.Name });
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Insertable(typeAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(typeUpdateList).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        public async Task<bool> UpdateGoodsType(GoodstypeUpdateDto dto)
        {
            try
            {
                var model=SqlSugarDb.Db.Queryable<BPA_GoodsType>().Where(x => x.Id == dto.Id).First();
                if(model==null) throw Oops.Oh("工艺分类不存在");
                if (dto.Pid == null) dto.Pid = "0";
                model.Name = dto.Name;
                model.Pid = dto.Pid;
                model.Sort = dto.Sort;
                var res = await SqlSugarDb.Db.Updateable(model).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }
        public async Task<bool> DelGoodsType(GoodstypeDelDto dto)
        {
            try
            {
                var res = 0;
                SqlSugarDb.Db.BeginTran();
                var typeList = await SqlSugarDb.Db.Queryable<BPA_GoodsType>().ToListAsync();
                var goodList = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
                var attrList = await SqlSugarDb.Db.Queryable<BPA_GoodsAttribute>().ToListAsync();
                var attrValueList = await SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
                foreach (var id in dto.Ids)
                {
                    var type = typeList.FirstOrDefault(t => t.Id == id);
                    if (type == null)
                        throw Oops.Oh("工艺分类不存在");
                    var isUse = goodList.Any(t => t.GoodsTypeId == id);
                    if (isUse)
                        throw Oops.Oh($"工艺分类“{type.Name}”已使用");
                    var typeChildList = typeList.Where(t => t.Pid == id);
                    foreach(var typeChild in typeChildList)
                    {
                        var childIsUse = goodList.Any(t => t.GoodsTypeId == typeChild.Id);
                        if (childIsUse)
                            throw Oops.Oh($"工艺分类“{type.Name}”的子类“{typeChild.Name}”已使用");
                        var childAttrList = attrList.Where(t => t.GoodsTypeId == typeChild.Id).ToList();
                        var childAttrValueList = attrValueList.Where(t => childAttrList.Select(a => a.Id).Contains(t.GoodsAttributeId)).ToList();
                        await SqlSugarDb.Db.Deleteable(childAttrList).ExecuteCommandAsync();
                        await SqlSugarDb.Db.Deleteable(childAttrValueList).ExecuteCommandAsync();
                    }
                    var attributeList = attrList.Where(t => t.GoodsTypeId == id).ToList();
                    var attributeValueList = attrValueList.Where(t => attributeList.Select(a => a.Id).Contains(t.GoodsAttributeId)).ToList();
                    await SqlSugarDb.Db.Deleteable(attributeList).ExecuteCommandAsync();
                    await SqlSugarDb.Db.Deleteable(attributeValueList).ExecuteCommandAsync();
                    res += await SqlSugarDb.Db.Deleteable(type).ExecuteCommandAsync();
                }
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        public async Task<PageUtil> GetGoodsUintPage(GoodsUintQueryDto dto)
        {
            int total = new RefAsync<int>();
            var data = await SqlSugarDb.Db.Queryable<BPA_GoodsUint>()
                .Select(a => new GoodsUintViewDto()
                {
                    Id = a.Id.SelectAll()
                })
                .ToPageListAsync(dto.Current, dto.PageSize, total);
            return new PageUtil()
            {
                Total = total,
                Data = data
            };
        }
        public async Task<List<ResponseMessageDto>> AddGoodsUint(List<GoodsUintInsertDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var uintList = await SqlSugarDb.Db.Queryable<BPA_GoodsUint>().ToListAsync();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var uintAddList = new List<BPA_GoodsUint>();
                var uintUpdateList = new List<BPA_GoodsUint>();
                foreach (var item in dto)
                {
                    var uintData = uintList.FirstOrDefault(t => t.Name == item.Name && t.Id != item.Id);
                    if (uintData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "商品单位上传失败：重复商品单位数据", Id = item.Id, Name = item.Name });
                    var uintInfo = uintList.FirstOrDefault(t => t.Id == item.Id);
                    if (uintInfo == null)
                    {
                        uintInfo = new BPA_GoodsUint
                        {
                            Id = item.Id,
                            Name = item.Name,
                            Remark = item.Remark,
                            CreateBy = userId
                        };
                        uintAddList.Add(uintInfo);
                    }
                    else
                    {
                        uintInfo.Name = item.Name;
                        uintInfo.Remark = item.Remark;
                        uintUpdateList.Add(uintInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "商品单位上传成功", Id = item.Id, Name = item.Name });
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Insertable(uintAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(uintUpdateList).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        public async Task<bool> UpdateGoodsUint(GoodsUintUpdateDto dto)
        {
            try
            {
                var model = SqlSugarDb.Db.Queryable<BPA_GoodsUint>().Where(x => x.Id == dto.Id).First();
                if (model == null) throw Oops.Oh("商品单位不存在");
                model.Name = dto.Name;
                model.Remark = dto.Remark;
                var res = await SqlSugarDb.Db.Updateable(model).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }
        public async Task<bool> DelGoodsUint(GoodsUintDelDto dto)
        {
            try
            {
                var res = 0;
                SqlSugarDb.Db.BeginTran();
                var uintList = await SqlSugarDb.Db.Queryable<BPA_GoodsUint>().ToListAsync();
                var goodList = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
                foreach (var id in dto.Ids)
                {
                    var uintData = uintList.FirstOrDefault(t => t.Id == id);
                    if (uintData == null)
                        throw Oops.Oh("商品单位不存在");
                    var isUse = goodList.Any(t => t.GoodsUintId == id);
                    if (isUse)
                        throw Oops.Oh($"商品单位“{uintData.Name}”已使用");
                    res += await SqlSugarDb.Db.Deleteable(uintData).ExecuteCommandAsync();
                }
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        private string GetNumber2(int Length = 10)
        {
            byte[] buffer = Guid.NewGuid().ToByteArray();
            var ram = BitConverter.ToInt64(buffer, 0);
            var str = string.Format("{0}", ram.ToString().Substring(0, Length));
            return str;
        }

        public async Task<PageUtil> GetGoodsClassify(GoodsClassifyQueryDto dto)
        {
            var total = new RefAsync<int>();
            var relationList = await SqlSugarDb.Db.Queryable<BPA_GoodsClassifyRelation>().ToListAsync();
            var data = await SqlSugarDb.Db.Queryable<BPA_GoodsClassify>()
                .Select(a => new GoodsClassifyViewDto
                {
                    Id = a.Id.SelectAll()
                })
                .Mapper(t =>
                {
                    t.GoodsIdList = relationList.Where(m => m.ClassifyId == t.Id).Select(m => m.GoodsId).ToList();
                }).ToPageListAsync(dto.Current, dto.PageSize, total);
            return new PageUtil()
            {
                Total = total,
                Data = data
            };
        }

        public async Task<List<ResponseMessageDto>> AddGoodsClassify(List<GoodsClassifyInsertDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var classifyList = await SqlSugarDb.Db.Queryable<BPA_GoodsClassify>().ToListAsync();
                var relationList = await SqlSugarDb.Db.Queryable<BPA_GoodsClassifyRelation>().ToListAsync();
                var goodsList = await SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().ToListAsync();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var classifyAddList = new List<BPA_GoodsClassify>();
                var classifyUpdateList = new List<BPA_GoodsClassify>();
                var relationAddList = new List<BPA_GoodsClassifyRelation>();
                var relationUpdateList = new List<BPA_GoodsClassifyRelation>();
                foreach (var item in dto)
                {
                    var classifyData = classifyList.FirstOrDefault(t => t.Name == item.Name && t.Id != item.Id);
                    if(classifyData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "商品分类上传失败：重复商品分类数据", Id = item.Id, Name = item.Name });
                    var classifyInfo = classifyList.FirstOrDefault(t => t.Id == item.Id);
                    foreach (var good in item.GoodsList)
                    {
                        var goodsData = goodsList.FirstOrDefault(t => t.Id == good.GoodsId);
                        if (goodsData == null)
                            repeatList.Add(new ResponseMessageDto { Code = 30002, Message = "商品分类上传失败：未找到商品数据", Id = good.GoodsId, Name = good.GoodsName });
                        var goodRelation = relationList.FirstOrDefault(t => t.ClassifyId == item.Id && t.GoodsId == good.GoodsId);
                        if (goodRelation != null)
                        {
                            goodRelation.CreateAt = DateTime.Now;
                            relationUpdateList.Add(goodRelation);
                        }
                        else
                        {
                            relationAddList.Add(new BPA_GoodsClassifyRelation
                            {
                                Id = Guid.NewGuid().ToString(),
                                ClassifyId = item.Id,
                                GoodsId = good.GoodsId,
                                CreateBy = userId
                            });
                        }
                        successList.Add(new ResponseMessageDto { Code = 30000, Message = "商品分类的商品上传成功", Id = good.GoodsId, Name = good.GoodsName });
                    }
                    if (classifyInfo == null)
                    {
                        classifyInfo = new BPA_GoodsClassify
                        {
                            Id = item.Id,
                            Name = item.Name,
                            Sort = item.Sort,
                            Remark = item.Remark,
                            CreateBy = userId
                        };
                        classifyAddList.Add(classifyInfo);
                    }
                    else
                    {
                        classifyInfo.Name = item.Name;
                        classifyInfo.Sort = item.Sort;
                        classifyInfo.Remark = item.Remark;
                        classifyUpdateList.Add(classifyInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "商品分类上传成功", Id = item.Id, Name = item.Name });
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Insertable(relationAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Insertable(classifyAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(classifyUpdateList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(relationUpdateList).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return successList;
            }
            catch (Exception e)
            {

                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
    }
}
