﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.KitChenManage.Core;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.CheckService.Services;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Services;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Core;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;
using BPA.SAAS.Manage.Core.Product;
using Microsoft.AspNetCore.Components.Forms;
using Newtonsoft.Json;
using NPOI.SS.Formula.Functions;
using System.Drawing.Drawing2D;
using System.Text;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Services
{
    public class MaterialService : IMaterialServices, ITransient
    {
        private readonly ICheckServices _checkServices;
        private readonly IThirdpartyPushService _thirdpartyPushService;
        public MaterialService(ICheckServices checkServices, IThirdpartyPushService thirdpartyPushService)
        {
            _checkServices = checkServices;
            _thirdpartyPushService = thirdpartyPushService;
        }

        /// <summary>
        /// 分页查询物料
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<MaterialDto>>> GetMaterialPageList(MaterialPageInputDto inputDto)
        {


            int total = new RefAsync<int>();
            var data = await SqlSugarDb.Db.Queryable<BPA_Batching, BPA_BatchingType, BPA_BatchingUint>((a, b, c) =>
                new JoinQueryInfos(JoinType.Left, a.Batching_Type == b.Id,
                    JoinType.Left, a.StockUint == c.Id))
                .Where((a, b, c) => a.IsDeleted == 0)
                .WhereIF(!string.IsNullOrEmpty(inputDto.Name), (a, b, c) => a.Batching_Name.Contains(inputDto.Name))
                .Select((a, b, c) => new MaterialDto()
                {
                    Id = a.Id,
                    Code = a.Code,
                    Name = a.Batching_Name,
                    TypeId = b.Id,
                    TypeName = b.Name,
                    UintId = c.Id,
                    UintName = c.Name,

                }).ToPageListAsync(inputDto.Current, inputDto.PageSize, total);

            return new PageUtil<List<MaterialDto>>()
            {
                Total = total,
                Data = data
            };
        }


        /// <summary>
        /// 添加物料
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<List<ResponseMessageDto>> AddMaterial(List<MaterialCreateDto> inputDto)
        {
            try
            {
                var userId = await _checkServices.GetUserId(App.HttpContext.Request.Headers["key"].ToString());
                var oldUintList = await SqlSugarDb.Db.Queryable<BPA_BatchingUint>().ToListAsync();
                var defaultUintData = oldUintList.FirstOrDefault(t => t.Name == "g" || t.Name == "默认单位");
                var defaultUint = new DefaultUintDto();
                if (defaultUintData != null)
                {
                    defaultUint.Id = defaultUintData.Id;
                    defaultUint.Name = defaultUintData.Name;
                }
                else
                {
                    var dUint = new BPA_BatchingUint
                    {
                        Id = Guid.NewGuid().ToString(),
                        Name = "默认单位",
                        CreateBy = userId
                    };
                    await SqlSugarDb.Db.Insertable(dUint).ExecuteCommandAsync();
                    defaultUint.Id = dUint.Id;
                    defaultUint.Name = dUint.Name;
                }
                SqlSugarDb.Db.Ado.BeginTran();
                var batchingList = await SqlSugarDb.Db.Queryable<BPA_Batching>().ToListAsync();
                var typeList = await SqlSugarDb.Db.Queryable<BPA_BatchingType>().ToListAsync();
                var uintList = await SqlSugarDb.Db.Queryable<BPA_BatchingUint>().ToListAsync();
                var typeIdList = new List<string>();
                var uintIdList = new List<string>();
                var repeatList = new List<ResponseMessageDto>();
                var successList = new List<ResponseMessageDto>();
                var typeAddList = new List<BPA_BatchingType>();
                var typeUpdateList = new List<BPA_BatchingType>();
                var uintAddList = new List<BPA_BatchingUint>();
                var uintUpdateList = new List<BPA_BatchingUint>();
                var batchingAddList = new List<BPA_Batching>();
                var batchingUpdateList = new List<BPA_Batching>();
                foreach (var item in inputDto)
                {
                    var batchingData = batchingList.FirstOrDefault(t => t.Batching_Name == item.Name && t.Id != item.Id);
                    if (batchingData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "物料上传失败：重复物料数据", Id = item.Id, Name = item.Name });
                    var typeData = typeList.FirstOrDefault(t => t.Name == item.TypeName && t.Id != item.TypeId);
                    if (typeData != null)
                        repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "物料类型上传失败：重复物料类型数据", Id = item.TypeId, Name = item.TypeName });
                    if (string.IsNullOrWhiteSpace(item.UintId))
                    {
                        item.UintId = defaultUint.Id;
                        item.UintName = defaultUint.Name;
                    }
                    else
                    {
                        var uintData = uintList.FirstOrDefault(t => t.Name == item.UintName && t.Id != item.UintId);
                        if (uintData != null)
                            repeatList.Add(new ResponseMessageDto { Code = 30001, Message = "物料单位上传失败：重复物料单位数据", Id = item.UintId, Name = item.UintName });
                    }
                    var typeInfo = typeList.FirstOrDefault(x => x.Id == item.TypeId);
                    if (typeInfo == null)
                    {
                        if (!typeIdList.Contains(item.TypeId))
                        {
                            typeInfo = new BPA_BatchingType()
                            {
                                Id = item.TypeId,
                                Name = item.TypeName,
                                CreateBy = userId
                            };
                            typeAddList.Add(typeInfo);
                            typeIdList.Add(item.TypeId);
                        }
                    }
                    else
                    {
                        typeInfo.Name = item.TypeName;
                        typeUpdateList.Add(typeInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "物料类型上传成功", Id = item.TypeId, Name = item.TypeName });
                    var uintInfo = uintList.FirstOrDefault(x => x.Id == item.UintId);
                    if (uintInfo == null)
                    {
                        if (!uintIdList.Contains(item.UintId))
                        {
                            uintInfo = new BPA_BatchingUint()
                            {
                                Id = item.UintId,
                                Name = item.UintName,
                                CreateBy = userId
                            };
                            uintAddList.Add(uintInfo);
                            uintIdList.Add(item.UintId);
                        }
                    }
                    else
                    {
                        uintInfo.Name = item.UintName;
                        uintUpdateList.Add(uintInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "物料单位上传成功", Id = item.UintId, Name = item.UintName });
                    var code = GenerateRandomString();
                    while (batchingList.Any(t => t.Code == code))
                    {
                        code = GenerateRandomString();
                    }
                    var batchingInfo = batchingList.FirstOrDefault(t => t.Id == item.Id);
                    if (batchingInfo == null)
                    {
                        batchingInfo = new BPA_Batching()
                        {
                            Id = item.Id,
                            Aittribute = 0,
                            Batching_Name = item.Name,
                            Batching_Type = item.TypeId,
                            Code = code,
                            Price = 0,
                            Specs = "",
                            outstockUint = item.UintId,
                            Status = CommonStatus.ENABLE,
                            StockUint = item.UintId,
                            TypeID = item.TypeId,
                            IsDeleted = 0,
                            CreateBy = userId
                        };
                        batchingAddList.Add(batchingInfo);
                    }
                    else
                    {
                        batchingInfo.Batching_Name = item.Name;
                        batchingInfo.Batching_Type = item.TypeId;
                        batchingInfo.TypeID = item.TypeId;
                        batchingInfo.outstockUint = item.UintId;
                        batchingInfo.StockUint = item.UintId;
                        batchingUpdateList.Add(batchingInfo);
                    }
                    successList.Add(new ResponseMessageDto { Code = 30000, Message = "物料上传成功", Id = item.Id, Name = item.Name });
                }
                if (repeatList.Count > 0)
                    return repeatList;
                await SqlSugarDb.Db.Insertable(typeAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(typeUpdateList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Insertable(uintAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(uintUpdateList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Insertable(batchingAddList).ExecuteCommandAsync();
                await SqlSugarDb.Db.Updateable(batchingUpdateList).ExecuteCommandAsync();
                SqlSugarDb.Db.Ado.CommitTran();
                return successList;
                #region 下发数据到设备（后厨弃用）

                //var data = new List<PushDataBatchingDto>();
                //foreach (var item in list)
                //{
                //    data.Add(new PushDataBatchingDto()
                //    {
                //        Aittribute = item.Aittribute,
                //        AutoKey = item.AutoKey,
                //        Code = item.Code,
                //        ForeignKeyRe = item.ForeignKeyRe,
                //        Id = item.Id,
                //        Name = item.Batching_Name,
                //        netrecovery = item.netrecovery,
                //        outstockUint = item.outstockUint,
                //        Price = item.Price,
                //        proportion = item.proportion,
                //        Specs = item.Specs,
                //        StockUint = item.StockUint,
                //        StockUintName = item.StockUint,
                //        TypeID = item.TypeID,
                //    });
                //}

                //if (inputDto.IsPush)
                //{
                //    await _thirdpartyPushService.AddPushRecordAndPushDevice<MaterialCreateDto>(inputDto, 2, JsonConvert.SerializeObject(list.Select(x => x.Id).ToList()),
                //        JsonConvert.SerializeObject(data));
                //}


                #endregion
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除物料
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> DelMaterial(DelMaterialDto inputDto)
        {
            try
            {
                var materialData = await SqlSugarDb.Db.Queryable<BPA_Batching>().Where(x => inputDto.Ids.Contains(x.Id)).ToListAsync();
                if (materialData == null)
                    throw Oops.Oh("物料不存在");
                var res = await SqlSugarDb.Db.Deleteable(materialData).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 修改物料
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateMateria(List<MaterialUpdateDto> inputDto)
        {
            try
            {
                SqlSugarDb.Db.Ado.BeginTran();
                var batchingList = await SqlSugarDb.Db.Queryable<BPA_Batching>().ToListAsync();
                var typeList = await SqlSugarDb.Db.Queryable<BPA_BatchingType>().ToListAsync();
                var uintList = await SqlSugarDb.Db.Queryable<BPA_BatchingUint>().ToListAsync();
                var typeIdList = new Dictionary<string, string>();
                var uintIdList = new Dictionary<string, string>();
                foreach (var item in inputDto)
                {
                    //1.物料单位查询
                    var typeId = string.Empty;
                    var typeData = typeList.FirstOrDefault(x => x.Name == item.TypeName);
                    if (typeData == null)
                    {
                        if (!typeIdList.ContainsKey(item.TypeName))
                        {
                            typeData = new BPA_BatchingType()
                            {
                                Id = Guid.NewGuid().ToString(),
                                Name = item.TypeName
                            };
                            SqlSugarDb.Db.Insertable(typeData).ExecuteCommand();
                            typeIdList.Add(item.TypeName, typeData.Id);
                            typeId = typeData.Id;
                        }
                        else
                            typeId = typeIdList[item.TypeName];
                    }
                    else
                        typeId = typeData.Id;
                    //2.单位查询
                    var uintId = string.Empty;
                    var uintData = uintList.FirstOrDefault(x => x.Name == item.UintName);
                    if (uintData == null)
                    {
                        if (!uintIdList.ContainsKey(item.UintName))
                        {
                            uintData = new BPA_BatchingUint()
                            {
                                Id = Guid.NewGuid().ToString(),
                                Name = item.UintName
                            };
                            SqlSugarDb.Db.Insertable(uintData).ExecuteCommand();
                            uintIdList.Add(item.UintName, uintData.Id);
                            uintId = uintData.Id;
                        }
                        else
                            uintId = uintIdList[item.UintName];
                    }
                    else
                        uintId = uintData.Id;
                    var bathingData = batchingList.FirstOrDefault(t => t.Id == item.Id);
                    if (bathingData == null)
                        throw Oops.Oh($"物料“{item.Name}”不存在");
                    bathingData.Batching_Name = item.Name;
                    bathingData.Batching_Type = typeId;
                    bathingData.outstockUint = uintId;
                    bathingData.StockUint = uintId;
                    bathingData.TypeID = typeId;
                    SqlSugarDb.Db.Updateable(bathingData).ExecuteCommand();
                    #region 下发数据到设备（后厨弃用）

                    //var data = new PushDataBatchingDto()
                    //{
                    //    Aittribute = materialData.Aittribute,
                    //    AutoKey = materialData.AutoKey,
                    //    Code = materialData.Code,
                    //    ForeignKeyRe = materialData.ForeignKeyRe,
                    //    Id = materialData.Id,
                    //    Name = materialData.Batching_Name,
                    //    netrecovery = materialData.netrecovery,
                    //    outstockUint = materialData.outstockUint,
                    //    Price = materialData.Price,
                    //    proportion = materialData.proportion,
                    //    Specs = materialData.Specs,
                    //    StockUint = materialData.StockUint,
                    //    StockUintName = materialData.StockUint,
                    //    TypeID = materialData.TypeID,
                    //};
                    //if (inputDto.IsPush)
                    //{
                    //    await _thirdpartyPushService.AddPushRecordAndPushDevice<MaterialUpdateDto>(inputDto, 2, materialData.Id,
                    //        JsonConvert.SerializeObject(data));
                    //}


                    #endregion
                }
                SqlSugarDb.Db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        private static Random random = new();
        public static string GenerateRandomString()
        {
            const string letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            const string digits = "0123456789";
            StringBuilder result = new();
            for (int i = 0; i < 4; i++)
            {
                result.Append(letters[random.Next(letters.Length)]);
            }
            for (int i = 0; i < 4; i++)
            {
                result.Append(digits[random.Next(digits.Length)]);
            }
            return result.ToString();
        }

        /// <summary>
        /// 分页查询物料分类
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<MaterialTypeDto>>> GetMaterialTypePageList(MaterialPageInputDto inputDto)
        {
            int total = new RefAsync<int>();
            var data = await SqlSugarDb.Db.Queryable<BPA_BatchingType>()
                .WhereIF(!string.IsNullOrEmpty(inputDto.Name), a => a.Name.Contains(inputDto.Name))
                .Select(a => new MaterialTypeDto()
                {
                    Id = a.Id,
                    Name = a.Name
                }).ToPageListAsync(inputDto.Current, inputDto.PageSize, total);
            return new PageUtil<List<MaterialTypeDto>>()
            {
                Total = total,
                Data = data
            };
        }
    }
}
