﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsAttribute;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsType;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Application.Org.Dtos.Organize;
using BPA.SAAS.Manage.Application.Org.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Comm.Util;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Org;
using Furion.LinqBuilder;
using StackExchange.Profiling.Internal;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class GoodsTypeService : IGoodsTypeService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public GoodsTypeService(ISqlSugarClient db)
        {
            _db = db;
        }
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetGoodsTypePage(GoodsTypeQueryDto dto)
        {
            List<IConditionalModel> conModels = new List<IConditionalModel>();
            //string groupId = App.User.FindFirst(ClaimConst.GroupId)?.Value;
            if (!string.IsNullOrEmpty(dto.Name))
            {
                conModels.Add(new ConditionalModel() { FieldName = "Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            }
            if (!string.IsNullOrEmpty(dto.Status))
            {
                conModels.Add(new ConditionalModel() { FieldName = "Status", ConditionalType = ConditionalType.Equal, FieldValue = dto.Status });
            }
            int total = new RefAsync<int>();
            var res = await _db.Queryable<BPA_GoodsType>()
                .Where(conModels)
                .OrderBy(a => a.Sort, OrderByType.Asc)
                .Select(it => new GoodsTypeTree() { Id = it.Id, Name = it.Name, Pid = it.Pid, Sort = it.Sort, Remark = it.Remark, Status = it.Status })
                .Mapper(x =>
                {
                    var attrViewList = _db.Queryable<BPA_GoodsAttribute>()
                .Where(a => a.IsDeleted == 0)
                .WhereIF(!x.Id.IsNullOrWhiteSpace(), a => a.GoodsTypeId.Contains(x.Id))
                .OrderBy(a => a.Sort, OrderByType.Asc)
                .Select(a => new GoodsAttributeView
                {
                    Id = a.Id,
                    AttributeName = a.AttributeName,
                    GoodsTypeId = a.GoodsTypeId,
                    Sort = a.Sort,
                })
                .Mapper(x =>
                {
                    var list = _db.Queryable<BPA_GoodsAttributeValue>().Where(p => p.GoodsAttributeId == x.Id).OrderBy(x => x.Sort).ToList();
                    x.AttributeValueList = list;
                }).ToList();
                    x.GoodsAttributeViewList = attrViewList;
                })
                .ToTreeAsync(it => it.Children, it => it.Pid, 0);
            //.ToPageList(dto.Current, dto.PageSize, ref total);
            List<GoodsTypeTree> list = new();
            if (res != null)
            {
                list = res.Skip((dto.Current - 1) * dto.PageSize).Take(dto.PageSize).ToList();
            }
            PageUtil util = new PageUtil()
            {
                Total = res == null ? 0 : res.Count,
                Data = list
            };
            return util;
        }
        /// <summary>
        /// 添加工艺分类
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoodsType(GoodsTypeDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var res = 0;
                if (string.IsNullOrWhiteSpace(dto.Id))
                {
                    var old = await _db.Queryable<BPA_GoodsType>().FirstAsync(t => t.Name == dto.Name);
                    if (old != null)
                        throw Oops.Oh($"已存在{dto.Name}，添加失败！");
                    var newType = new BPA_GoodsType
                    {
                        Pid = dto.Pid ?? "0",
                        Name = dto.Name,
                        Remark = dto.Remark,
                        Sort = dto.Sort,
                        Status = CommonStatus.ENABLE,
                    };
                    var typeEntity = await _db.Insertable(newType).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                    res++;
                    if (dto.AttributeList != null)
                        res += await HandleGoodsAttribute(typeEntity, dto.AttributeList);
                    _db.Ado.CommitTran();
                    return res > 0;
                }
                else
                {
                    var typeEntity = _db.Queryable<BPA_GoodsType>().First(it => it.Id == dto.Id);
                    if (null == typeEntity)
                    {
                        return false;
                    }
                    typeEntity.Name = dto.Name;
                    typeEntity.Pid = dto.Pid ?? "0";
                    typeEntity.Sort = dto.Sort;
                    typeEntity.Remark = dto.Remark;
                    typeEntity.Status = (CommonStatus)Enum.ToObject(typeof(CommonStatus), dto.Status);
                    await _db.Updateable(typeEntity).IgnoreColumns(ignoreAllNullColumns: true).ExecuteCommandAsync();
                    res++;
                    if (dto.AttributeList != null)
                        res += await HandleGoodsAttribute(typeEntity, dto.AttributeList);
                    _db.Ado.CommitTran();
                    return res > 0;
                }
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        public async Task<int> HandleGoodsAttribute(BPA_GoodsType typeEntity, List<AttributeDto> AttributeList)
        {
            try
            {
                var res = 0;
                var attributeList = await _db.Queryable<BPA_GoodsAttribute>().Where(it => it.GoodsTypeId == typeEntity.Id).ToListAsync();
                var attributeValueList = await _db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
                var actionList = await _db.Queryable<BPA_GoodsTechnologyAction>().Select(t => t.GoodsAttributeId).Distinct().ToListAsync();
                if (attributeList.Count > 0)
                {
                    foreach (var oldAttribute in attributeList)
                    {
                        var oldValueList = attributeValueList.Where(it => it.GoodsAttributeId == oldAttribute.Id).ToList();
                        foreach (var _ in oldValueList.SelectMany(oldValue => actionList.Where(action => action.Contains(oldValue.Id)).Select(action => new { }).Select(_ => new { })))
                        {
                            throw Oops.Oh("该工艺分类属性已存在工艺，请删除对应工艺才能修改！");
                        }

                        if (oldValueList.Count > 0)
                            res += await _db.Deleteable(oldValueList).ExecuteCommandAsync();
                    }
                    res += await _db.Deleteable(attributeList).ExecuteCommandAsync();
                }
                if (AttributeList != null)
                {
                    foreach (var i in AttributeList)
                    {
                        var attribute = new BPA_GoodsAttribute
                        {
                            GoodsTypeId = typeEntity.Id,
                            AttributeName = i.AttributeName,
                            Sort = i.Sort
                        };
                        var attributeEntity = await _db.Insertable(attribute).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                        var valueList = new List<BPA_GoodsAttributeValue>();
                        if (i.AttributeValueList != null)
                        {
                            foreach (var j in i.AttributeValueList)
                            {
                                var value = new BPA_GoodsAttributeValue
                                {
                                    GoodsAttributeId = attributeEntity.Id,
                                    AttributeValue = j.AttributeValue,
                                    Sort = j.Sort
                                };
                                valueList.Add(value);
                            }
                        }
                        if(valueList.Count > 0)
                            res += await _db.Insertable(valueList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    }
                }
                return res;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 更新工艺分类
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoodsType(GoodsTypeDto dto)
        {
            // 查询数据库中是否存在未删除的工艺分类
            var resEntity = _db.Queryable<BPA_GoodsType>().First(it => it.Id == dto.Id);
            if (null == resEntity)
            {
                return false;
            }

            resEntity.Name = dto.Name;
            resEntity.Pid = dto.Pid;
            resEntity.Sort = dto.Sort;
            resEntity.Remark = dto.Remark;

            resEntity.Status = (CommonStatus)Enum.ToObject(typeof(CommonStatus), dto.Status);

            var res = await _db.Updateable(resEntity).IgnoreColumns(ignoreAllNullColumns: true).ExecuteCommandAsync();
            return res > 0;
        }
        public async Task<List<dynamic>> GetGoodsTypeList_alm()
        {
            var groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            var res = await _db.Queryable<BPA_GoodsType>().Where(x => x.GroupId == groupId).Select<dynamic>(x => new

            {
                id = x.Id,
                name = x.Name,
                pid = x.Pid,
            }).ToListAsync();
            return res;
        }
        /// <summary>
        /// 删除工艺分类
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<bool> DelGoodsType(string Id)
        {
            try
            {
                _db.Ado.BeginTran();
                var res = 0;
                var ids = Id.Split(",");
                // 查询数据库中是否存在未删除的工艺分类
                var check = _db.Queryable<BPA_GoodsInfo>().Any(it => ids.Contains(it.GoodsTypeId));
                if (check)
                {
                    throw Oops.Oh("该类已经使用无法删除");
                }
                var resChildList = _db.Queryable<BPA_GoodsType>().Where(t => ids.Contains(t.Pid)).ToList();
                var childIds = resChildList.Select(a => a.Id).ToList();
                var childCheck = _db.Queryable<BPA_GoodsInfo>().Any(it => childIds.Contains(it.GoodsTypeId));
                if (childCheck)
                {
                    throw Oops.Oh("该类已经使用无法删除");
                }
                var resList = _db.Queryable<BPA_GoodsType>().Where(it => ids.Contains(it.Id)).ToList();
                var attributeList = _db.Queryable<BPA_GoodsAttribute>().Where(it => ids.Contains(it.GoodsTypeId)).ToList();
                foreach (var attribute in attributeList)
                {
                    var valueList = _db.Queryable<BPA_GoodsAttributeValue>().Where(it => it.GoodsAttributeId == attribute.Id).ToList();
                    res += await _db.Deleteable(valueList).ExecuteCommandAsync();
                }
                res += await _db.Deleteable(attributeList).ExecuteCommandAsync();
                res += await _db.Deleteable(resChildList).ExecuteCommandAsync();
                res += await _db.Deleteable(resList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return res > 0;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 查询工艺分类树结构
        /// </summary>
        /// <returns></returns>
        public async Task<dynamic> GetGoodsTypeTree()
        {
            var dataScopeList = new List<string>();
            var groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value;
            var orgs = await _db.Queryable<BPA_GoodsType>().WhereIF(dataScopeList.Count > 0, u => dataScopeList.Contains(u.Id))
                                                        .Where(u => u.Status == CommonStatus.ENABLE && u.IsDeleted == 0 && u.GroupId == groupId).OrderBy(u => u.Sort)
                                                        .Select(u => new OrgTreeNode
                                                        {
                                                            key = u.Id,
                                                            parentId = u.Pid,
                                                            title = u.Name,
                                                            value = u.Id.ToString(),
                                                            weight = u.Sort,
                                                            Type = 0,
                                                        }).ToListAsync();

            return new TreeBuildUtil<OrgTreeNode>().Build(orgs);

        }
    }
}
